from simpleserver.server import start_server
from sdk.config import Config
if __name__ == "__main__":
    Config(True, 'avGYo8TAFL', 'fc6e4922bda4148ab4a734f5acbe58a7ce3a684a', 'http://vpca-phoenix-buttonwood-service-1.vm.elenet.me:8080/callback',
           "https://open-api-sandbox-shop.alpha.elenet.me")
    start_server()



# -*- coding: utf-8 -*-
import json
import web
from sdk.util.callback_signature_util import CallbackValidationUtil
from sdk.oauth.oauth_client import OAuthClient
from sdk.config import Config
from sdk.protocol.rpc_client import RpcClient
import uuid

urls = (
        '/listenMessage', 'ListenMessage',
        '/callback', 'CallBack',
        '/getInfo', 'GetUserInfo',
        )


class ListenMessage(object):
    def GET(self):
        return '{"message":"ok"}'

    def POST(self):
        message = web.data()
        if not CallbackValidationUtil.is_valid_message(message):
            return '{"message":"InvalidSignature"}'
        return '{"message":"ok"}'


class CallBack(object):
    def GET(self):
        render = web.template.render('templates')
        params = web.input()
        if not params:
            return render.demo()
        auth_code = params.code
        if auth_code:
            oauth_client = OAuthClient(Config.get_app_key(), Config.get_secret(), Config.get_callback_url())
            response = oauth_client.get_token_by_auth_code(auth_code)
            #response = oauth_client.get_token_in_client_credentials()
            token = json.loads(response)['access_token']
            rpc_client = RpcClient(token)
            user_info = rpc_client.call("eleme.user.getUser", {})
            user_id = user_info['userId']
            f = open("mapping.txt", "a")
            f.write(str(user_id)+token+'\n')
            f.close()
            return render.redirect(user_id,user_info['authorizedShops'][0]['id'])
        else:
            return render.demo()


class GetUserInfo(object):
    def POST(self):
        data = web.data()
        print data
        result = {}
        params = json.loads(data)
        user_id = params['userId']
        shop_id = params['shopId']
        file = open("mapping.txt", "r")
        while 1:
            line = file.readline()
            if not line:
                oauth_client = OAuthClient(Config.get_app_key(), Config.get_secret(), Config.get_callback_url())
                auth_url = oauth_client.get_auth_url(str(uuid.uuid4()), 'all')
                result['authUrl'] = auth_url
                return result
            if user_id == line.split(':')[0]:
                token = line.split(':')[1]
                rpc_client = RpcClient(token)
                user_info = rpc_client.call("eleme.user.getUser", {})
                result['userInfo'] = user_info
                authorized_shops = user_info['authorizedShops']
                for shop in authorized_shops:
                    if shop['id'] == int(shop_id):
                        shop_info = rpc_client.call("eleme.shop.getShop", {"shopId": int(shop_id)})
                        result['shopInfo'] = shop_info
                        return result
        return result


def start_server():
    app = web.application(urls, globals())
    app.run()

if __name__ == "__main__":
    # config = Config(True, 'avGYo8TAFL', 'fc6e4922bda4148ab4a734f5acbe58a7ce3a684a', None,
    #                 "https://open-api-sandbox-shop.alpha.elenet.me")
    Config(True, 'ja7rOj8Zkj', '4573a5f70a349661a8ce6489696e07d0e7d5a9b9',
           'http://vpca-phoenix-buttonwood-service-1.vm.elenet.me:8080/callback', "https://open-api-sandbox-shop.alpha.elenet.me")
    start_server()



# -*- coding: utf-8 -*-
import base64
import urllib2
import json
import uuid


def generate_code():
    json_file = open("example.json", "w")
    # doc_json=json_file.readline()
    #doc_json=get_doc('ORDER','3d2e4546485058b73a0eade67e251d47926745b5')
    doc_json=get_last_doc()
    # json_file.write(doc_json)
    # file.close()
    list_doc=json.loads(doc_json)['result']
    for result in list_doc:
        interfaceName = result['className']
        module_name = str(interfaceName).split(".")[1].capitalize()
        apis_file = open("../sdk/apis/" + module_name.lower() + "_service.py", "w")
        apis_file.write("# -*- coding: utf-8 -*-\n\n\n")
        apis_file.write("class " + module_name + "Service:\n\n")
        apis_file.write("    __client = None\n\n")
        apis_file.write("    def __init__(self, client):\n        self.__client = client\n\n")
        methods = result['classMethods']
        for method in methods:
            action = method['methodName']
            methodParams = method['methodParams']
            join_params = ''
            join_params_json = '{'
            for param in methodParams:
                field_name = str(param["fieldName"])
                join_params = join_params + ", " + camel_to_underline(field_name)
                if (param == methodParams[-1]):
                    join_params_json = join_params_json + '"' + field_name + '": ' + camel_to_underline(field_name)
                else:
                    join_params_json = join_params_json + '"' + field_name + '": ' + camel_to_underline(
                        field_name) + ", "
            join_params_json = join_params_json + "}"
            apis_file.write("    def " + camel_to_underline(str(action).split(".")[2]) + "(self" + join_params + "):\n")
            apis_file.write('        return self.__client.call("' + action + '", ' + join_params_json + ')\n\n')
        apis_file.close()


def camel_to_underline(camel_format):
    underline_format=''
    if isinstance(camel_format, str):
        for _s_ in camel_format:
            underline_format += _s_ if _s_.islower() else '_'+_s_.lower()
    return underline_format


def get_doc(type, version):
    headers = {
        "Content-Type": "application/json; charset=utf-8"
    }

    remote_url='https://open-api.shop.ele.me/operating/invoke?method=ShuttleService.getESDoc'
    request_json={
        "id":  str(uuid.uuid4()),
        "method": "getESDoc",
        "service": "ShuttleService",
        "params": {"serviceType": type, "docVersion": version},
        "metas": {"appName": "melody", "appVersion": "4.4.0"},
        "ncp": "2.0.0"
    }
    request = urllib2.Request(remote_url, json.dumps(request_json).encode('utf-8'), headers)
    response = urllib2.urlopen(request)
    result = response.read()
    return result.decode('utf-8')


def get_last_doc():
    headers = {
        "Content-Type": "application/json; charset=utf-8"
    }

    remote_url='https://open-api.shop.ele.me/operating/invoke?method=ShuttleService.getESDoc'
    request_json={
        "id":  str(uuid.uuid4()),
        "method": "getLatestESDoc",
        "service": "ShuttleService",
        "params": {},
        "metas": {"appName":"melody","appVersion":"4.4.0"},
        "ncp":"2.0.0"
    }
    request = urllib2.Request(remote_url, json.dumps(request_json).encode('utf-8'), headers)
    response = urllib2.urlopen(request)
    result = response.read()
    return result.decode('utf-8')

if __name__ == '__main__':
    generate_code()


# -*- coding: utf-8 -*-

import base64
import urllib2
import urllib
from sdk.config import Config


class OAuthClient:

    def __init__(self, app_key, secret, callback_url=None):
        self.client_id = app_key
        self.secret = secret
        self.callback_url = callback_url
        self.request_url = Config.get_access_token_url()
        self.authorize_url = Config.get_authorize_url()

    # 根据key和secret获取token（客户端授权模式）
    def get_token_in_client_credentials(self):
        data = {"grant_type": "client_credentials"}
        return self.do_request(data)

    # 构造授权URL（授权码模式）
    def get_auth_url(self, state, scope):
        url = self.authorize_url
        client_id = self.client_id
        response_type = "code"
        callback_url = urllib2.quote(self.callback_url)
        return "%s?response_type=%s&client_id=%s&state=%s&redirect_uri=%s&scope=%s" % (
            url, response_type, client_id, state, callback_url, scope)

    # 根据auth_code获取token（授权码模式）
    def get_token_by_auth_code(self, code):
        callback_url = self.callback_url
        data = {"grant_type": "authorization_code",
                "code": code,
                "redirect_uri": callback_url,
                "client_id": self.client_id}

        return self.do_request(data)

    # 根据refresh_token刷新获取token（授权码模式）
    def get_token_by_refresh_token(self, refresh_token, scope):
        data = {"grant_type": "refresh_token",
                "refresh_token": refresh_token,
                "scope": scope}

        return self.do_request(data)

    def do_request(self, body):
        req = urllib2.Request(self.request_url, urllib.urlencode(body).encode("ascii"),
                                     self.get_headers())
        response = urllib2.urlopen(req)
        result = response.read()
        return result.decode()

    def get_headers(self):
        slice = u"{}:{}".format(self.client_id, self.secret)
        value = base64.b64encode(bytes(slice))
        headers = {
            "Authorization": "Basic " + value.decode(),
            "Content-Type": "application/x-www-form-urlencoded; charset=utf-8"}
        return headers




# -*- coding: utf-8 -*-


class OrderService:

    __client = None

    def __init__(self, client):
        self.__client = client

    def get_order(self, order_id):
        return self.__client.call("eleme.order.getOrder", {"orderId": order_id})

    def mget_orders(self, order_ids):
        return self.__client.call("eleme.order.mgetOrders", {"orderIds": order_ids})

    def confirm_order(self, order_id):
        return self.__client.call("eleme.order.confirmOrder", {"orderId": order_id})

    def cancel_order(self, order_id, type, remark):
        return self.__client.call("eleme.order.cancelOrder", {"orderId": order_id, "type": type, "remark": remark})

    def agree_refund(self, order_id):
        return self.__client.call("eleme.order.agreeRefund", {"orderId": order_id})

    def disagree_refund(self, order_id, reason):
        return self.__client.call("eleme.order.disagreeRefund", {"orderId": order_id, "reason": reason})




# -*- coding: utf-8 -*-

import hashlib
from sdk.config import Config
import json


class CallbackValidationUtil:

    @staticmethod
    def is_valid_message(message):
        message = json.loads(message.decode("utf-8"))
        signature = message['signature']
        data = []
        for k, v in message.items():
            data.append(u"{}={}".format(k, v))
        sorted_msg = sorted(data)
        string = "".join(sorted_msg)
        string = u"{}{}".format(string, Config.get_secret())
        hash_value = hashlib.md5(string.encode("utf-8")).hexdigest().upper()
        return hash_value == signature



# -*- coding: utf-8 -*-


class ProductService:

    __client = None

    def __init__(self, client):
        self.__client = client

    def item(self, category_id):
        return self.__client.call("eleme.product.item.getItemsByCategoryId", {"categoryId": category_id})

    def item(self, item_id):
        return self.__client.call("eleme.product.item.getItem", {"itemId": item_id})

    def item(self, item_ids):
        return self.__client.call("eleme.product.item.batchGetItems", {"itemIds": item_ids})

    def item(self, category_id, properties):
        return self.__client.call("eleme.product.item.createItem", {"categoryId": category_id, "properties": properties})

    def item(self, category_id, items):
        return self.__client.call("eleme.product.item.batchCreateItems", {"categoryId": category_id, "items": items})

    def item(self, item_id, category_id, properties):
        return self.__client.call("eleme.product.item.updateItem", {"itemId": item_id, "categoryId": category_id, "properties": properties})

    def item(self, spec_ids):
        return self.__client.call("eleme.product.item.batchFillStock", {"specIds": spec_ids})

    def item(self, spec_ids):
        return self.__client.call("eleme.product.item.batchClearStock", {"specIds": spec_ids})

    def item(self, spec_ids):
        return self.__client.call("eleme.product.item.batchOnShelf", {"specIds": spec_ids})

    def item(self, spec_ids):
        return self.__client.call("eleme.product.item.batchOffShelf", {"specIds": spec_ids})

    def item(self, item_id):
        return self.__client.call("eleme.product.item.removeItem", {"itemId": item_id})

    def item(self, item_ids):
        return self.__client.call("eleme.product.item.batchRemoveItems", {"itemIds": item_ids})

    def category(self, shop_id):
        return self.__client.call("eleme.product.category.getShopCategories", {"shopId": shop_id})

    def category(self, category_id):
        return self.__client.call("eleme.product.category.getCategory", {"categoryId": category_id})

    def category(self, shop_id, name, description):
        return self.__client.call("eleme.product.category.createCategory", {"shopId": shop_id, "name": name, "description": description})

    def category(self, category_id, name, description):
        return self.__client.call("eleme.product.category.updateCategory", {"categoryId": category_id, "name": name, "description": description})

    def category(self, category_id):
        return self.__client.call("eleme.product.category.removeCategory", {"categoryId": category_id})

    def upload_image(self, image):
        return self.__client.call("eleme.file.uploadImage", {"image": image})

    def upload_image_with_remote_url(self, url):
        return self.__client.call("eleme.file.uploadImageWithRemoteUrl", {"url": url})

    def get_uploaded_url(self, hash):
        return self.__client.call("eleme.file.getUploadedUrl", {"hash": hash})




# -*- coding: utf-8 -*-


class MessageService:

    __client = None

    def __init__(self, client):
        self.__client = client

    def get_non_reached_messages(self, app_id):
        return self.__client.call("eleme.message.getNonReachedMessages", {"appId": app_id})

    def get_non_reached_o_messages(self, app_id):
        return self.__client.call("eleme.message.getNonReachedOMessages", {"appId": app_id})




# -*- coding: utf-8 -*-

import hashlib
import json
import time
import urllib2
import uuid

from sdk.exception.access_denied_exception import AccessDeniedException
from sdk.exception.business_exception import BusinessException
from sdk.exception.exceed_limit_exception import ExceedLimitException
from sdk.exception.illegal_request_exception import IllegalRequestException
from sdk.exception.invalid_signature_exception import InvalidSignatureException
from sdk.exception.invalid_timestamp_exception import InvalidTimestampException
from sdk.exception.method_not_allowed_exception import MethodNotAllowedException
from sdk.exception.permission_denied_exception import PermissionDeniedException
from sdk.exception.system_exception import SystemException
from sdk.exception.unauthorized_exception import UnauthorizedException
from sdk.exception.validation_failed_exception import ValidationFailedException
from sdk.config import Config


class RpcClient:
    def __init__(self, token):
        self.token = token
        self.app_key = Config.get_app_key()
        self.secret = Config.get_secret()
        self.remote_url = Config.get_api_server_url()

    def call(self, action, parameters):
        protocol = {
            "nop": "1.0.0",
            "id": str(uuid.uuid4()),
            "action": action,
            "token": self.token,
            "metas": {
                "app_key": self.app_key,
                "timestamp": int(time.time())
            },
            "params": parameters
        }

        protocol['signature'] = self.generate_signature(protocol)

        result = self.post(protocol)
        return self.parse_result(result)

    def parse_result(self, result):
        response = json.loads(result)
        if response['error'] != None:
            code = response['error']['code']
            message = response['error']['message']
            if code == "SERVER_ERROR":
                raise SystemException(message)
            elif code == "ILLEGAL_REQUEST":
                raise IllegalRequestException(message)
            elif code == "UNAUTHORIZED":
                raise UnauthorizedException(message)
            elif code == "ACCESS_DENIED":
                raise AccessDeniedException(message)
            elif code == "METHOD_NOT_ALLOWED":
                raise MethodNotAllowedException(message)
            elif code == "PERMISSION_DENIED":
                raise PermissionDeniedException(message)
            elif code == "INVALID_SIGNATURE":
                raise InvalidSignatureException(message)
            elif code == "INVALID_TIMESTAMP":
                raise InvalidTimestampException(message)
            elif code == "VALIDATION_FAILED":
                raise ValidationFailedException(message)
            elif code == "EXCEED_LIMIT":
                raise ExceedLimitException(message)
            else:
                raise BusinessException(message)

        return response['result']

    def generate_signature(self, protocol):
        merged = protocol['metas'].copy()
        merged.update(protocol['params'])
        data = []
        for key in merged:
            value = merged[key]
            data.append(key + "=" + json.dumps(value, ensure_ascii=False, separators=(',', ':')))

        sorted_data = sorted(data)
        splice = protocol['action'] + protocol['token'] + "".join(sorted_data) + self.secret
        hash_value = hashlib.md5(splice.encode("UTF-8")).hexdigest()
        return hash_value.upper()

    def post(self, data):
        headers = {
            "Content-Type": "application/json; charset=utf-8"
        }
        # print(self.__remote_url,data)
        request = urllib2.Request(self.remote_url, json.dumps(data).encode('utf-8'), headers)
        response = urllib2.urlopen(request)
        result = response.read()
        return result.decode('utf-8')



# -*- coding: utf-8 -*-
class Config:
    __sandbox = False
    # your app_key
    __app_key = None
    # your secret
    __secret = None
    # your callback url
    __callback_url = None

    __base_url = None

    def __init__(self, sandbox, app_key, secret, callback_url=None, base_url=None):
        Config.__sandbox = sandbox
        Config.__app_key = app_key
        Config.__secret = secret
        Config.__callback_url = callback_url
        Config.__base_url = base_url

    @staticmethod
    def get_app_key():
        return Config.__app_key

    @staticmethod
    def get_secret():
        return Config.__secret

    @staticmethod
    def get_callback_url():
        return Config.__callback_url

    @staticmethod
    def get_server_url():
        if not Config.__base_url:
            return Config.__sandbox and "https://open-api-sandbox.shop.ele.me" or "https://open-api.shop.ele.me"
        else:
            return Config.__base_url

    @staticmethod
    def get_access_token_url():
        return Config.get_server_url() + "/token"

    @staticmethod
    def get_api_server_url():
        return Config.get_server_url() + "/api/v1/"

    @staticmethod
    def get_authorize_url():
        return Config.get_server_url() + "/authorize"



from sdk.config import Config
from sdk.oauth.oauth_client import OAuthClient
from sdk.protocol.rpc_client import RpcClient
from sdk.apis.user_service import UserService
import json

config = Config(True, 'avGYo8TAFL', 'fc6e4922bda4148ab4a734f5acbe58a7ce3a684a', None, "https://open-api-sandbox-shop.alpha.elenet.me")

#config = Config(True, 'ja7rOj8Zkj', '4573a5f70a349661a8ce6489696e07d0e7d5a9b9', None, "https://open-api-sandbox-shop.alpha.elenet.me")


def get_token():
    oauth_client = OAuthClient(config.get_app_key(), config.get_secret())
    return oauth_client.get_token_in_client_credentials()


def get_token_by_auth_code():
    oauth_client = OAuthClient(config.get_app_key(), config.get_secret(),'http://vpca-phoenix-buttonwood-service-1.vm.elenet.me:8080/callback')
    auth_url = oauth_client.get_token_by_auth_code('864e688f17b624f866ab509009ef782e')
    print auth_url


def user_get_user(token):
    client = RpcClient(token)
    user_service = UserService(client)
    response=user_service.get_user()
    print response

if __name__ == '__main__':
    token = json.loads(get_token())['access_token']
    #print token
    #########test user_service#########
    user_get_user(token)
    #get_token_by_auth_code()





# -*- coding: utf-8 -*-


class ShopService:

    __client = None

    def __init__(self, client):
        self.__client = client

    def get_shop(self, shop_id):
        return self.__client.call("eleme.shop.getShop", {"shopId": shop_id})

    def update_shop(self, shop_id, properties):
        return self.__client.call("eleme.shop.updateShop", {"shopId": shop_id, "properties": properties})

    def mget_shop_status(self, shop_ids):
        return self.__client.call("eleme.shop.mgetShopStatus", {"shopIds": shop_ids})






from setuptools import setup, find_packages

setup(
    name='eleme.openapi.sdk',
    version='0.0.1.13',
    keywords=('eleme', 'openapi'),
    #long_description=open('README.md').read(),
    description='eleme openapi python sdk',
    license='MIT License',
    package_data = {
        '': ['*.html'],
    },
    packages=find_packages(),
    #install_requires=['web>=0.38'],
    author='david',
    author_email='pengfei.zhu@ele.me',
)




# -*- coding: utf-8 -*-


class UserService:

    __client = None

    def __init__(self, client):
        self.__client = client

    def get_user(self):
        return self.__client.call("eleme.user.getUser", {})

